unit uMHA_Import;

{-
   This feature is designed so a new instrument can be added to the local Vista
   system. The imported instrument is contained in an encrypted file constructed
   with the Instrument Authoring Tool (IAT).


-}


interface

uses
  classes,
  Sysutils,
  graphics,
  Trpcb,
  fIA_dialogs, Dialogs,
  uIA_Display,
  uIA_Rule,
  uIA_Question,
  uIA_Instrument;

type
  TImport = class

  private
    function ImportAndSaveDescriptiveProperties(var aInstrument: TInstrument): string;
    function ImportQuestions(var aInstrument: TInstrument): string;
    function ImportIntroductions(var aInstrument: TInstrument): string;
    function ImportSections(var aInstrument: TInstrument): string;
    function ImportRules(aInstrument: TInstrument): string;
    function ImportScales(aInstrument: TInstrument): string;
    function ImportReport(aInstrument: TInstrument): string;
  //  function ConvertOldToNew(OldQuestionID: string): string;
    procedure FreeEverything;
    function CheckForProperVersion: Boolean;
    function ImportQuestionsIntrosToVista(aInstrument: TInstrument): Boolean;
    function SearchDisplays(WhichDisplay: TDisplay;
      IsAnswerDisplay: BOOLEAN): string;
    function UnencryptFile(aFileName: string): string;
  protected
  public
    function ImportInstrument(FullFileName: string): boolean;
    procedure WriteUpdateSeries(aSeries: string);
    function GetDisplayID(
      FontIn: TFont;
      AlignmentIn: TAlignment;
      LeftIn: Integer;
      IsAnswerDisplay: Boolean;
      Mask: string;
      Columns: Integer;
      Component: string): string;
  published
  end; { T_Import }

var
  Import                      : TImport;

implementation

uses
  uMHA_Const,
  uIA_Const,
  uIA_Utils,
  uIA_Display_OPS,
  uIA_Question_OPS,
  uIA_Introduction,
  uIA_SectionLabel,
  uIA_AvailableInstrument,
  uMHA_Instrument_OPS,
  uIA_Key,
  uIA_ScaleGroup,
  uIA_Scale,
  uIA_Scales_IO,
  uIA_SectionLabel_IO,
  uIA_Keys_IO,
  uIA_Question_IO,
  uMHA_Import_IO,
  fMHA_Progress,
  uMHA_DM,

  LbCipher, // from TurboPower, LockBox
  LbProc,
  LbClass;

const
  lcTempFileStr               : string = 'temp.txt';

var
  lvMax,
    lvProgress                : integer;
  lvImportStrList             : TStringList;
  lvInstrument                : TInstrument;

function TImport.CheckForProperVersion: Boolean;
begin
  Result := False;
  if Pos('Revised Oct 2009', lvImportStrList[0]) > 0 then
    Result := True;
end;

function TImport.UnencryptFile(aFileName: string): string;
const
  lcKey                       : LongInt = 2222222222;
begin
  Result := BAD_DATA;

  // Make sure the temp file is not already in use
  if FileExists(lcTempFileStr) then
    DeleteFile(lcTempFileStr);

  if FileExists(aFileName) then
    begin
      RNG32EncryptFile(aFileName,
        lcTempFileStr, lcKey);

      result := OK;
    end;

end;


function TImport.ImportInstrument(FullFileName: string): boolean;
var
  Reply                       : string;
begin
  result := false;
  lvProgress := 0;

  lvInstrument := TInstrument.Create;
  lvInstrument.Clear;
  lvImportStrList := TStringList.Create;

  // Un-encrypt the file
  Reply := UnencryptFile(FullFileName);
  if Reply = BAD_DATA then
    begin
      MsgDlgFrm('The imported instrument was not saved to Vista. ' + #13 + #13 +
        FullFileName + ' is not not in a proper format to be un-encrypted.',
        MHA_Caption, mtInformation, [mbOK], 0);
      lvInstrument.free;
      lvImportStrList.free;
      Exit;
    end;

  lvImportStrList.LoadFromFile(lcTempFileStr);
  lvMax := lvImportStrList.Count;

  if lvMax <= 1 then
    begin
      MsgDlgFrm('The imported instrument was not saved to Vista. ' + #13 + #13 +
        'The file, ' + FullFileName +
        ', is not in the proper format to be un-encrypted.',
        MHA_Caption, mtInformation, [mbOK], 0);
      lvInstrument.free;
      lvImportStrList.free;
      Exit;
    end;

  lvInstrument.InstrumentName := Piece(lvImportStrList[1], U, 1);

  // The file name and the lvInstrument.InstrumentName are the same except the
  // encrypted file name starts with an 'e'.
  if (ExtractFileName(FullFileName)) <> ('e' + lvInstrument.InstrumentName + '.iie') then
    begin
      MsgDlgFrm('Sorry, the file is not encrypted properly. ' + #13 +
        'This instrument cannot be imported.',
        MHA_Caption, mtInformation, [mbOK], 0);
      lvInstrument.free;
      lvImportStrList.free;
      Exit;
    end;

  // Make sure this file is something useful.
  if NumberOfPieces(lvImportStrList[1], U) > 30 then
  else
    begin
      MsgDlgFrm('The imported instrument was not saved to Vista. ' + #13 + #13 +
        'The file does not appear to be structured properly: ' + #13 +
        gvNameOfInstrument,
        MHA_Caption, mtInformation, [mbOK], 0);
      lvInstrument.free;
      lvImportStrList.free;
      Exit;
    end;

  if not CheckForProperVersion then
    begin
      MsgDlgFrm('The file was built with an earlier version of IAT and ' +
        'cannot be read by this application.',
        MHA_Caption, mtInformation, [mbOK], 0);
      lvInstrument.free;
      lvImportStrList.free;
      Exit;
    end;

  if lvInstrument.InstrumentName = '' then
    begin
      MsgDlgFrm('Sorry, the name of the instrument to be imported is blank. ' + #13 +
        '(This is not the same name as the name of the file.)' + #13 +
        'The import file is likely corrupted.',
        MHA_Caption, mtInformation, [mbOK], 0);
      lvInstrument.free;
      lvImportStrList.free;
      Exit;
    end;

  frmMHA_Progress := TfrmMHA_Progress.Create(nil);
  frmMHA_Progress.SetupProgressBar(lvInstrument.InstrumentName + ': Saving name. . .',
    lvMax);
  frmMHA_Progress.IncreaseProgressBar;

  // Write new name to Vista
  Reply := WriteInstrumentName(lvInstrument.InstrumentName, DataModule1.RPCBroker1);

  if (POS('duplicate', reply) > 0) then
    begin
      MsgDlgFrm('Sorry, there is an instrument with this name already ' +
        'installed on Vista: ' + lvInstrument.InstrumentName + #13 + #13 +
        'If it is not listed among the instruments, it may be discontinued, ' +
        'retired, or perhaps its license has expired.',
        MHA_Caption, mtInformation, [mbOK], 0);
      FreeEverything;
      Exit;
    end;

  if reply <> OK then
    begin
      MsgDlgFrm('The instrument was not saved to Vista. ' +
        'There was a problem saving its name: ' + lvInstrument.InstrumentName + #13 + #13 +
        reply,
        MHA_Caption, mtInformation, [mbOK], 0);
      FreeEverything;
      Exit;
    end;

  frmMHA_Progress.SetupProgressBar(lvInstrument.InstrumentName +
    ': Importing properties. . .',
    lvMax);

  // Load the instrument's IEN: needed later
  Instrument_OPS.LoadDescriptiveProperties(lvInstrument);

  Reply := ImportAndSaveDescriptiveProperties(lvInstrument);
  if Reply <> OK then
    begin
      MsgDlgFrm(reply,
        MHA_Caption, mtInformation, [mbOK], 0);
      FreeEveryThing;
      Exit;
    end;

  lvInstrument.ListOfQuestions.Clear;
  lvInstrument.ListOfIntroductions.Clear;
  lvInstrument.ListOfSections.Clear;
  lvMax := lvMax + lvInstrument.NumberOfQuestions;

  frmMHA_Progress.SetupProgressBar(lvInstrument.InstrumentName +
    ': Importing questions. . .',
    lvMax);

  Reply := ImportQuestions(lvInstrument);
  if Reply <> OK then
    begin
      MsgDlgFrm(reply,
        MHA_Caption, mtInformation, [mbOK], 0);
      FreeEverything;
      Exit;
    end;

  Reply := ImportIntroductions(lvInstrument);
  if Reply <> OK then
    begin
      MsgDlgFrm(reply,
        MHA_Caption, mtInformation, [mbOK], 0);
      FreeEverything;
      Exit;
    end;

  if not ImportQuestionsIntrosToVista(lvInstrument) then
  // Saves Questions & Introductions: error messages presented there
    begin
      FreeEverything;
      Exit;
    end;

  frmMHA_Progress.SetupProgressBar(lvInstrument.InstrumentName +
    ': Importing sections. . .',
    lvMax);

  Reply := ImportSections(lvInstrument);
  if Reply <> OK then
    begin
      MsgDlgFrm(reply,
        MHA_Caption, mtInformation, [mbOK], 0);
      FreeEverything;
      Exit;
    end;

  frmMHA_Progress.SetupProgressBar(lvInstrument.InstrumentName +
    ': Importing rules. . .',
    lvMax);
  Reply := ImportRules(lvInstrument);
  if Reply <> OK then
    begin
      MsgDlgFrm(reply,
        MHA_Caption, mtInformation, [mbOK], 0);
      FreeEverything;
      Exit;
    end;

  frmMHA_Progress.SetupProgressBar(lvInstrument.InstrumentName +
    ': Importing scales. . .',
    lvMax);
  Reply := ImportScales(lvInstrument);
  if Reply <> OK then
    begin
      MsgDlgFrm(reply,
        MHA_Caption, mtInformation, [mbOK], 0);
      FreeEverything;
      Exit;
    end;

  frmMHA_Progress.SetupProgressBar(lvInstrument.InstrumentName +
    ': Importing report. . .',
    lvMax);
  Reply := ImportReport(lvInstrument);
  if Reply <> OK then
    begin
      MsgDlgFrm(reply,
        MHA_Caption, mtInformation, [mbOK], 0);
      FreeEverything;
      Exit;
    end;

  result := true;

  MsgDlgFrm(lvInstrument.InstrumentName + ' has been imported successfully.',
    MHA_Caption, mtInformation, [mbOK], 0);

  FreeEverything;
end;

procedure TImport.FreeEverything;
begin
  lvImportStrList.Free;
  lvInstrument.Free;

  if frmMHA_Progress <> nil then
    frmMHA_Progress.Free;

  if FileExists(lcTempFileStr) then
    DeleteFile(lcTempFileStr);
end;


function TImport.ImportQuestionsIntrosToVista(aInstrument: TInstrument): Boolean;
var
  i                           : Integer;
  TempIntroID,
    reply                     : string;
begin
  Result := False;

  for i := 0 to aInstrument.ListOfQuestions.Count - 1 do
    begin
      frmMHA_Progress.IncreaseProgressBar;
      with TQuestion(aInstrument.ListOfQuestions[i]) do
        begin
          // Save introduction.
          if TIntroduction(aInstrument.ListOfIntroductions[i]).IsNew then
            begin
              Reply := WriteIntroText(TIntroduction(aInstrument.ListOfIntroductions[i]).iText,
                '', DataModule1.RPCBroker1); // Returns "OK^IEN" when working properly
              if Pos(OK, Reply) > 0 then
                begin
                  TIntroduction(aInstrument.ListOfIntroductions[i]).ID :=
                    Piece(Reply, U, 2);
                  TempIntroID := TIntroduction(aInstrument.ListOfIntroductions[i]).ID;
                end
              else
                begin
                  MsgDlgFrm('Error saving introduction: ' + qCaption + CRLF + CRLF +
                    Reply, MHA_Caption, mtError, [mbOK], 0);
                  Exit;
                end;
            end
          else
            TIntroduction(aInstrument.ListOfIntroductions[i]).ID := TempIntroID;

          // Save multiple-choice items.
          if (ResponseType = rtMultiChoice) or
            (ResponseType = rtCheckList) or
            (ResponseType = rtTrackBar) then
            begin
              Reply := WriteMultiChoiceItems(TQuestion(aInstrument.ListOfQuestions[i]),
                DataModule1.RPCBroker1); // Question_IO

              if Pos(OK, Reply) = 0 then // Not OK, indicates error
                begin
                  MsgDlgFrm('Error saving multiple choice items: ' + qCaption + CRLF + CRLF +
                    Reply, MHA_Caption, mtError, [mbOK], 0);
                  Exit;
                end;
            end;

      // Blanks are not stored in Vista fields; "@" will delete the field (not
      // the record) thus a blank.
          if Designator = '' then
            Designator := '@';

          // Save the question: different calls for brand new vs. existing
          Reply := WriteQuestion(aInstrument,
            TIntroduction(aInstrument.ListOfIntroductions[i]).ID,
            TIntroduction(aInstrument.ListOfIntroductions[i]).Display.DisplayID,
            TQuestion(aInstrument.ListOfQuestions[i]),
            DataModule1.RPCBroker1);

          if Pos(OK, Reply) = 0 then // Not OK, indicates error
            begin
              MsgDlgFrm('Error saving question ' + qCaption + CRLF + CRLF +
                Reply, MHA_Caption, mtError, [mbOK], 0);
              Exit;
            end;

          //lvOldNewQuestionID.ValueFromIndex[i] := QuestionID;
          //**
          //lvoldnewquestionid.SaveToFile('index.txt');
        end; // with
    end; //for i

  result := True;
end;

function TImport.ImportAndSaveDescriptiveProperties(var aInstrument: TInstrument): string;
var
  i                           : Integer;
begin
  Result := OK;

  with aInstrument do
    begin
      i := 2;
      // Name is already present
      PrintTitle := Piece(lvImportStrList[1], U, i);
      Inc(i);
      Version := Piece(lvImportStrList[1], U, i);
      Inc(i);
      RequiresSignature := StrToBool(Piece(lvImportStrList[1], U, i));
      Inc(i);
      Reference := Piece(lvImportStrList[1], U, i);
      Inc(i);
      RequiresLicense := StrToBool(Piece(lvImportStrList[1], U, i));
      Inc(i);
      IsOperational := 'Y';
      Inc(i);
      LicenseIsCurrent := StrToBool(Piece(lvImportStrList[1], U, i));
      Inc(i);
      IsASI := StrToBool(Piece(lvImportStrList[1], U, i));
      Inc(i);
      IsLegacy := StrToBool(Piece(lvImportStrList[1], U, i));
      Inc(i);
      NumberOfQuestions := StrToIntDef(Piece(lvImportStrList[1], U, i), 0);
      Inc(i);
      DaysToRestart := StrToIntDef(Piece(lvImportStrList[1], U, i), 0);
      Inc(i);
      IEN := Piece(lvImportStrList[1], U, i);
      Inc(i);
      Publisher := Piece(lvImportStrList[1], U, i);
      Inc(i);
      Author := Piece(lvImportStrList[1], U, i);
      Inc(i);
      Copyright := Piece(lvImportStrList[1], U, i);
      Inc(i);
      PublicationDate := Piece(lvImportStrList[1], U, i);
      Inc(i);
      NormSample := Piece(lvImportStrList[1], U, i);
      Inc(i);
      R_Privilege := Piece(lvImportStrList[1], U, i);
      Inc(i);
      Purpose := Piece(lvImportStrList[1], U, i);
      Inc(i);
      TargetPopulation := Piece(lvImportStrList[1], U, i);
      Inc(i);
      EntryDate := Piece(lvImportStrList[1], U, i);
      Inc(i);
      EnteredBy := Piece(lvImportStrList[1], U, i);
      Inc(i);
      A_Privilege := Piece(lvImportStrList[1], U, i);
      Inc(i);
      LastEditedBy := Piece(lvImportStrList[1], U, i);
      Inc(i);
      LastEditDate := FormatDateTime('MM/dd/yyyy', Date);
      Inc(i);
      IsNationalTest := StrToBool(Piece(lvImportStrList[1], U, i));
      Inc(i);
      LastEditFMDate := Piece(lvImportStrList[1], U, i);
      Inc(i);
      SubmitToNationalDB := StrToBool(Piece(lvImportStrList[1], U, i));
      Inc(i);
      IsCopyrighted := StrToBool(Piece(lvImportStrList[1], U, i));
      Inc(i);
      WriteFullText := StrToBool(Piece(lvImportStrList[1], U, i));
      Inc(i);
      GenerateProgressNote := StrToBool(Piece(lvImportStrList[1], U, i));
      Inc(i);
      TIU_Title := Piece(lvImportStrList[1], U, i);
    end;

  Result := WriteInstrumentDescription(aInstrument, DataModule1.RPCBroker1);
  // Returns OK when working properly, otherwise an error statement.
end;



function TImport.ImportQuestions(var aInstrument: TInstrument): string;
var
  Count,
    i, j, k                   : Integer;
  AQuestion                   : TQuestion;

begin
  Result := OK;
  Count := 1;

  with Display_OPS, Question_OPS do
    begin
  // Start a 3; 0,1,2 for instrument object
      for i := 2 to lvImportStrList.Count - 1 do
        begin

      // String with Question data
          if Pos('Question_', lvImportStrList[i]) = 1 then
            begin
              j := 2;
              frmMHA_Progress.IncreaseProgressBar;
              AQuestion := TQuestion.Create;

              AQuestion.qText := Piece(lvImportStrList[i], U, j);
              Inc(j);
              AQuestion.QuestionID := Piece(lvImportStrList[i], U, j);
              //lvOldNewQuestionID.Add(AQuestion.QuestionID + '=');
              Inc(j);
              AQuestion.qMax := StrToIntDef(Piece(lvImportStrList[i], U, j), 0);
              Inc(j);
              AQuestion.IsRequired := StrToBool(Piece(lvImportStrList[i], U, j));
              Inc(j);
              AQuestion.qMin := StrToIntDef(Piece(lvImportStrList[i], U, j), 0);
              Inc(j);
              AQuestion.Designator := Piece(lvImportStrList[i], U, j);
              Inc(j);
              AQuestion.qHint := Piece(lvImportStrList[i], U, j);
              Inc(j);
              AQuestion.qCaption := Piece(lvImportStrList[i], U, j);
              Inc(j);
              AQuestion.IsLastQuestion := StrToBool(Piece(lvImportStrList[i], U, j));
              Inc(j);
              AQuestion.IsFirstQuestion := StrToBool(Piece(lvImportStrList[i], U, j));
              Inc(j);
              AQuestion.ResponseType := ConvertStringToResponseType
                (Piece(lvImportStrList[i], U, j));
              Inc(j);

          // Build Question Display
              AQuestion.Display.DisplayFont.Name := Piece(lvImportStrList[i], U, j); // Font.name
              Inc(j);
              AQuestion.Display.DisplayFont.Style := [];
              if Piece(lvImportStrList[i], U, j) = 'Y' then // Bold: boolean
                AQuestion.Display.DisplayFont.Style := [fsBold];
              Inc(j);
              if Piece(lvImportStrList[i], U, j) = 'Y' then // Italic: boolean
                AQuestion.Display.DisplayFont.Style :=
                  AQuestion.Display.DisplayFont.Style + [fsItalic];
              Inc(j);
              if Piece(lvImportStrList[i], U, j) = 'Y' then // Underline: boolean
                AQuestion.Display.DisplayFont.Style :=
                  AQuestion.Display.DisplayFont.Style + [fsUnderline];
              Inc(j);
          //Size: integer
              AQuestion.Display.DisplayFont.Size := StrToIntDef(
                Piece(lvImportStrList[i], U, j), 8);
              Inc(j);
          //Color: string
              AQuestion.Display.DisplayFont.Color := StringToColor(
                Piece(lvImportStrList[i], U, j));
              Inc(j);
           //Left: integer
              AQuestion.Display.DisplayLeft := StrToIntDef(
                Piece(lvImportStrList[i], U, j), 1);
              Inc(j);
          //Align: TAlignment
              AQuestion.Display.DisplayAlignment :=
                StrToAlignment(
                Piece(lvImportStrList[i], U, j));
              Inc(j);

              AQuestion.Display.DisplayID :=
                GetDisplayID(
                AQuestion.Display.DisplayFont,
                AQuestion.Display.DisplayAlignment,
                AQuestion.Display.DisplayLeft,
                False, NA, 0, NA);

              if AQuestion.Display.DisplayID = '' then
                AQuestion.Display.DisplayID := '@';

          // Build AQuestion.AnswerDisplay
              AQuestion.AnswerDisplay.DisplayFont.Name := Piece(lvImportStrList[i], U, j); // Font.name
              Inc(j);
              AQuestion.AnswerDisplay.DisplayFont.Style := [];
              if Piece(lvImportStrList[i], U, j) = 'Y' then // Bold: boolean
                AQuestion.AnswerDisplay.DisplayFont.Style := [fsBold];
              Inc(j);
              if Piece(lvImportStrList[i], U, j) = 'Y' then // Italic: boolean
                AQuestion.AnswerDisplay.DisplayFont.Style :=
                  AQuestion.AnswerDisplay.DisplayFont.Style + [fsItalic];
              Inc(j);
              if Piece(lvImportStrList[i], U, j) = 'Y' then // Underline: boolean
                AQuestion.AnswerDisplay.DisplayFont.Style :=
                  AQuestion.AnswerDisplay.DisplayFont.Style + [fsUnderline];
              Inc(j);
          //Size: integer
              AQuestion.AnswerDisplay.DisplayFont.Size := StrToIntDef(
                Piece(lvImportStrList[i], U, j), 8);
              Inc(j);
          //Color: string
              AQuestion.AnswerDisplay.DisplayFont.Color := StringToColor(
                Piece(lvImportStrList[i], U, j));
              Inc(j);
          //Left: integer
              AQuestion.AnswerDisplay.DisplayLeft := StrToIntDef(
                Piece(lvImportStrList[i], U, j), 1);
              Inc(j);
          // Width : integer
              AQuestion.AnswerDisplay.DisplayWidth := StrToIntDef(
                Piece(lvImportStrList[i], U, j), 100);
              Inc(j);
          //Align: TAlignment
              AQuestion.AnswerDisplay.DisplayAlignment :=
                StrToAlignment(
                Piece(lvImportStrList[i], U, j));
              Inc(j);
              AQuestion.AnswerDisplay.DisplayMask := Piece(lvImportStrList[i], U, j);
              Inc(j);
              AQuestion.AnswerDisplay.DisplayColumns := StrToIntDef(
                Piece(lvImportStrList[i], U, j), 1);
              Inc(j);
              AQuestion.AnswerDisplay.DisplayComponent :=
                StrToAnswerComponent(
                Piece(lvImportStrList[i], U, j));
              Inc(j);
              AQuestion.AnswerDisplay.DisplaySameLine :=
                StrToBool(Piece(lvImportStrList[i], U, j));
              Inc(j);

              AQuestion.AnswerDisplay.DisplayID :=
                GetDisplayID(
                AQuestion.AnswerDisplay.DisplayFont,
                AQuestion.AnswerDisplay.DisplayAlignment,
                AQuestion.AnswerDisplay.DisplayLeft,
                True,
                AQuestion.AnswerDisplay.DisplayMask,
                AQuestion.AnswerDisplay.DisplayColumns,
                AnswerComponentToStr(AQuestion.AnswerDisplay.DisplayComponent));

              if AQuestion.AnswerDisplay.DisplayID = '' then
                AQuestion.AnswerDisplay.DisplayID := '@';

          // Choices
              if Pos('Choices Follow', lvImportStrList[i]) > 0 then
                begin
                  Inc(j);
                  AQuestion.ChoiceIdentifier :=
                    ConvertStringToChoiceIdentifier(Piece(lvImportStrList[i], U, j));
                  Inc(j);
                  AQuestion.MultiChoice.NumberOfPicks := StrToInt(Piece(lvImportStrList[i], U, j));
                  Inc(j);
                  AQuestion.MultiChoice.NumberOfChoices := StrToInt(Piece(lvImportStrList[i], U, j));
                  Inc(j);

                  if AQuestion.MultiChoice.NumberOfChoices > 0 then
                    begin
                      for k := 1 to AQuestion.MultiChoice.NumberOfChoices do
                        begin
                          AQuestion.MultiChoice.Choices.Add(Piece(lvImportStrList[i], U, j));
                          Inc(j);
                        end;
                    end;
                end;

// ShowMessage('AQuestion.MultiChoice.NumberOfPicks '+ IntToStr(AQuestion.MultiChoice.NumberOfPicks) +#13+
// 'AQuestion.MultiChoice.NumberOfChoices ' +IntToStr(AQuestion.MultiChoice.NumberOfChoices));

// if AQuestion.MultiChoice.Choices.Count > 1 then
// ShowMessage(
// 'AQuestion.MultiChoice.Choices[0] ' +   AQuestion.MultiChoice.Choices[0] +#13+
// 'AQuestion.MultiChoice.Choices[1] ' +  AQuestion.MultiChoice.Choices[1]);

              AQuestion.InstrumentContentsSequenceField := IntToStr(Count * 10);
              Inc(Count);

          // Blanks are not stored in Vista fields; "@" will delete the field (not
          // the record) thus a blank.
              if AQuestion.Designator = '' then
                AQuestion.Designator := '@';

              aInstrument.ListOfQuestions.Add(AQuestion);
            end; // String with question data

        end; // for i; each string in import file
    end; // with Display_OPS
end;


(*function TImport.ConvertOldToNew(OldQuestionID: string): string;
begin
  Result := '';
  Result := lvOldNewQuestionID.Values[OldQuestionID];

end; *)

function TImport.ImportIntroductions(var aInstrument: TInstrument): string;
var
  i, j                        : Integer;
  AIntroduction               : TIntroduction;
begin
  Result := OK;

  with Display_OPS do
    begin
      for i := 2 to lvImportStrList.Count - 1 do
        begin

          if Pos('Introduction_', lvImportStrList[i]) = 1 then
            begin
              frmMHA_Progress.IncreaseProgressBar;
              with AIntroduction do
                begin
                  AIntroduction := TIntroduction.Create;
                  j := 2;

                  ID := Piece(lvImportStrList[i], U, j);
                  Inc(j);
                  iText := Piece(lvImportStrList[i], U, j);
                  Inc(j);
                  IsNew := StrToBool(Piece(lvImportStrList[i], U, j));
                  Inc(j);

                  Display.DisplayFont.Name := Piece(lvImportStrList[i], U, j); // Font.name
                  Inc(j);
                  Display.DisplayFont.Style := [];
                  if Piece(lvImportStrList[i], U, j) = 'Y' then // Bold: boolean
                    Display.DisplayFont.Style := [fsBold];
                  Inc(j);
                  if Piece(lvImportStrList[i], U, j) = 'Y' then // Italic: boolean
                    Display.DisplayFont.Style :=
                      Display.DisplayFont.Style + [fsItalic];
                  Inc(j);
                  if Piece(lvImportStrList[i], U, j) = 'Y' then // Underline: boolean
                    Display.DisplayFont.Style :=
                      Display.DisplayFont.Style + [fsUnderline];
                  Inc(j);
              //Size: integer
                  Display.DisplayFont.Size := StrToIntDef(
                    Piece(lvImportStrList[i], U, j), 8);
                  Inc(j);
              //Color: string
                  Display.DisplayFont.Color := StringToColor(
                    Piece(lvImportStrList[i], U, j));
                  Inc(j);
               //Left: integer
                  Display.DisplayLeft := StrToIntDef(
                    Piece(lvImportStrList[i], U, j), 1);
                  Inc(j);
              //Align: TAlignment
                  Display.DisplayAlignment :=
                    StrToAlignment(Piece(lvImportStrList[i], U, j));

                  Display.DisplayID :=
                    GetDisplayID(
                    Display.DisplayFont,
                    Display.DisplayAlignment,
                    Display.DisplayLeft,
                    False, '', 0, '');

                  if Display.DisplayID = '' then
                    Display.DisplayID := '@';

                  aInstrument.ListOfIntroductions.Add(AIntroduction);

                end;
            end;
        end;
    end; // with Display_OPS
end;


function TImport.ImportSections(var aInstrument: TInstrument): string;
var

  i, j                        : Integer;
  ASection                    : TSectionLabel;
  Reply                       : string;
begin
  Result := OK;

  with Display_OPS do
    begin
      for i := 2 to lvImportStrList.Count - 1 do
        begin

          if Pos('Section_', lvImportStrList[i]) = 1 then
            begin
              frmMHA_Progress.IncreaseProgressBar;
              with ASection do
                begin
                  ASection := TSectionLabel.Create;
                  ASection.Clear;
                  j := 2;

                  //QuestionID := ConvertOldToNew(Piece(lvImportStrList[i], U, j));
                  QuestionID := Piece(lvImportStrList[i], U, j);
                  Inc(j);
                  TabCaption := Piece(lvImportStrList[i], U, j);
                  Inc(j);
                  SectionCaption := Piece(lvImportStrList[i], U, j);
                  Inc(j);

                  Display.DisplayFont.Name := Piece(lvImportStrList[i], U, j); // Font.name
                  Inc(j);
                  Display.DisplayFont.Style := [];
                  if Piece(lvImportStrList[i], U, j) = 'Y' then // Bold: boolean
                    Display.DisplayFont.Style := [fsBold];
                  Inc(j);
                  if Piece(lvImportStrList[i], U, j) = 'Y' then // Italic: boolean
                    Display.DisplayFont.Style :=
                      Display.DisplayFont.Style + [fsItalic];
                  Inc(j);
                  if Piece(lvImportStrList[i], U, j) = 'Y' then // Underline: boolean
                    Display.DisplayFont.Style :=
                      Display.DisplayFont.Style + [fsUnderline];
                  Inc(j);
              //Size: integer
                  Display.DisplayFont.Size := StrToIntDef(
                    Piece(lvImportStrList[i], U, j), 8);
                  Inc(j);
              //Color: string
                  Display.DisplayFont.Color := StringToColor(
                    Piece(lvImportStrList[i], U, j));
                  Inc(j);
               //Left: integer
                  Display.DisplayLeft := StrToIntDef(
                    Piece(lvImportStrList[i], U, j), 1);
                  Inc(j);
              //Align: TAlignment
                  Display.DisplayAlignment :=
                    StrToAlignment(Piece(lvImportStrList[i], U, j));

                  Display.DisplayID :=
                    GetDisplayID(
                    Display.DisplayFont,
                    Display.DisplayAlignment,
                    Display.DisplayLeft,
                    False, '', 0, '');

                  if Display.DisplayID = '' then
                    Display.DisplayID := '@';

                  aInstrument.ListOfSections.Add(ASection);

                end; // POS
            end; // with
        end; // for i
    end; // with Display_OPS

  if aInstrument.ListOfSections.Count = 0 then
    Exit;

  for i := 0 to aInstrument.ListOfSections.Count - 1 do
    begin
      with TSectionLabel(aInstrument.ListOfSections[i]) do
        begin
          reply := WriteSection('' {Blank indicates new section},
            aInstrument.IEN,
            QuestionID,
            TabCaption,
            SectionCaption,
            Display.DisplayID,
            DataModule1.RPCBroker1);

          if Pos(OK, Reply) = 0 then
            begin
              MsgDlgFrm('Error saving Section ' + SectionCaption + CRLF + CRLF +
                Reply, MHA_Caption, mtError, [mbOK], 0);
              Result := 'ERROR';
              Exit;
            end;
        end;
    end;
end;


function TImport.ImportRules(aInstrument: TInstrument): string;
var
  x, i, j, k                  : Integer;
  reply,
  IEN,
    SkipQuestions             : string;
  ARule                       : TRule;

  function MyStrToBoolOp(ABoolOperatorString: string): TBoolOperatorType;
  begin
    Result := boAND; //the default value
    if (ABoolOperatorString = 'OR') then
      Result := boOR;
  end; //MyStrToBoolOp

  function MyStrToDataType(ADataTypeString: string): TValueDataType;
  begin
    //dtString, dtDate, dtNumber
    Result := dtString;
    if (ADataTypeString = DATE_STR) then
      Result := dtDate;
    if (ADataTypeString = NUMBER_STR) then
      Result := dtNumber;
    if (ADataTypeString = INTERVAL_STR) then
      Result := dtTimeInterval;
  end; //MyStrToDataType

  function MyStrToOperator(AOperatorString: string): TOperatorType;
  var
    tmpIndex                  : integer;
    tmpString                 : string;
  begin
    //toEQ, toLT, toLE, toGT, toGE, toNE
   //  OPERATOR_STRINGS = 'Equals^Does not equal^Is less than^Is greater than^'
   //    + 'Equals or is less than^Equals or is greater than';

    tmpString := AOperatorString;
    tmpIndex := PieceIndex(OPERATOR_STRINGS, tmpString, U);

    case tmpIndex of
      2: Result := toNE;
      3: Result := toLT;
      4: Result := toGT;
      5: Result := toLE;
      6: Result := toGE;
    else
      Result := toEQ;
    end; //case

  end; //MyStrToSecondOp

begin
  Result := OK;
  IEN := '';
  ARule := TRule.Create;

  for i := 2 to lvImportStrList.Count - 1 do
    begin
      // String with Rule data
      if Pos('Rule_', lvImportStrList[i]) = 1 then
        begin
          IEN := Piece(lvImportStrList[i], U, 2);
          frmMHA_Progress.IncreaseProgressBar;
          j := 2;
          ARule.Clear;
(*
          lvExportStrList.Add('Rule_' + IntToStr(i) + U +
            DataTypeToString(FirstValueDataType) + U +
            QuestionOperatorToStr(TRule(aInstrument.ListOfRules[i]), True) + U +
            DataTypeToString(SecondValueDataType) + U +
            QuestionOperatorToStr(TRule(aInstrument.ListOfRules[i]), False) + U +
            BoolOperatorToString(TRule(aInstrument.ListOfRules[i])) + U +
            FirstQuestionID + U +
            FirstValue + U +
            SecondQuestionID + U +
            SecondValue + U +
            MessageText + U +
            'Skip*' + SkipQuestionIDs);*)
          ARule.FirstValueDataType := MyStrToDataType(Piece(lvImportStrList[i], U, j));
          Inc(j);
          ARule.FirstOperator := MyStrToOperator(Piece(lvImportStrList[i], U, j));
          Inc(j);
          ARule.SecondValueDataType := MyStrToDataType(Piece(lvImportStrList[i], U, j));
          Inc(j);
          ARule.SecondOperator := MyStrToOperator(Piece(lvImportStrList[i], U, j));
          Inc(j);
          ARule.BoolOperator := MyStrToBoolOp(Piece(lvImportStrList[i], U, j));
          Inc(j);
          ARule.FirstQuestionID := Piece(lvImportStrList[i], U, j);
          //ARule.FirstQuestionID := ConvertOldToNew(Piece(lvImportStrList[i], U, j));
          //**
          //ShowMessage('Old ' + Piece(lvImportStrList[i], U, j) + #13 + ARule.FirstQuestionID);
          Inc(j);
          ARule.FirstValue := Piece(lvImportStrList[i], U, j);
          Inc(j);
          ARule.SecondQuestionID := Piece(lvImportStrList[i], U, j);
          //ARule.SecondQuestionID := ConvertOldToNew(Piece(lvImportStrList[i], U, j));
          Inc(j);
          ARule.SecondValue := Piece(lvImportStrList[i], U, j);
          Inc(j);
          ARule.MessageText := Piece(lvImportStrList[i], U, j);

          Reply := WriteRule(ARule, DataModule1.RPCBroker1);
          if Pos('ERROR', reply) > 0 then
            begin
              Result := Reply;
              Exit;
            end;

          if Piece(Reply, U, 2) <> '' then
            ARule.ID := Piece(Reply, U, 2);

          Reply := WriteRuleInstrument(
            ARule.ID,
            ARule.FirstQuestionID,
            aInstrument.IEN,
            DataModule1.RPCBroker1);
          if Pos('ERROR', reply) > 0 then
            begin
              Result := Reply;
              Exit;
            end;

          SkipQuestions := '';
          x := Pos('Skip*', lvImportStrList[i]);
          SkipQuestions := Copy(lvImportStrList[i], x, Length(lvImportStrList[i]));

          Reply := WriteSkippedQuestions(aInstrument.IEN,
            ARule.ID, SkipQuestions,
            DataModule1.RPCBroker1);
 (*
          OldSkipQuestions := '';
          NewSkipQuestions := '';
          x := Pos('Skip*', lvImportStrList[i]);
          OldSkipQuestions := Copy(lvImportStrList[i], x, Length(lvImportStrList[i]));
          x := NumberofPieces(OldSkipQuestions, U);

          if x > 1 then
            for k := 2 to x do
              begin
                NewSkipQuestions := NewSkipQuestions +
                  ConvertOldToNew(Piece(OldSkipQuestions, U, k)) + U;
              end;

          Reply := WriteSkippedQuestions(aInstrument.IEN,
            ARule.ID, NewSkipQuestions,
            DataModule1.RPCBroker1);
 *)
          if Reply <> OK then
            begin
              Result := 'Skip Question: ' + Reply;
              Exit;
            end;
        end; // POS
    end;

  ARule.Free;
end;


function TImport.ImportScales(aInstrument: TInstrument): string;
var
  i, j                        : Integer;
  TempScaleGroupID,
    TempScaleID,
    Reply                     : string;
  AScaleGroup                 : TScaleGroup;
  AScale                      : TScale;
  AKey                        : TKey;

  function GetGroupScaleID(GroupName: string): string;
  var
    ii                        : Integer;
    SubStr,
      SubSubStr,
      TempStr                 : string;
  begin
    Result := '';

    TempStr := GetScalesFromVista(aInstrument.InstrumentName,
      DataModule1.RPCBroker1);
    // Results look like this.
    // Group1= 100000^AllanS 1^Allan's Sizes^1^Ego Strength^0^5^50^10^30^40~
    // Scale1= 100000^100000^1^Ego Size^ES~
    // Scale2= 100001^100000^2^Shoe Size^Sz~
    // Group2= 100005^AllanS 1^Bob^1^^0^10^100^-1^-1^0~
    // Group3= 100001^AllanS 1^Allan's Bank Statement^2^Cash^22^4^88^40^60^70~
    // Scale3= 100002^100001^1^Cash Flow~
    // Scale4= 100003^100001^2^DebT^D~
    // Group4= 100007^AllanS 1^Bob^100^^0^10^100^-1^-1^0  //First record is unique~
    if TempStr = '' then
      Exit;

    if TempStr = BAD_DATA then
      begin
        Result := BAD_DATA;
        Exit;
      end;

    for ii := 1 to NumberOfPieces(TempStr, RECORD_END) do
      begin
        SubStr := Piece(TempStr, RECORD_END, ii);

        if Pos('Group', SubStr) > 0 then
          begin
    //  GroupX= [1] SCALEGROUP ID^
    //          [2] INSTRUMENT NAME^
    //          [3] SCALEGROUP NAME^
    //          [4] GROUP SEQUENCE^
    //          [5-11] ORDINATE TITLE^MIN^INCREMENT^MAX^GRID1^GRID2^GRID3

            SubSubStr := Piece(SubStr, EQ, 2);
            if Piece(SubSubStr, U, 3) = GroupName then
              Result := Piece(SubSubStr, U, 1);
          end;
      end;
  end;

  function GetScaleID(AName: string): string;
  var
    ii                        : Integer;
    SubStr,
      SubSubStr,
      TempStr                 : string;
  begin
    Result := '';

    TempStr := GetScalesFromVista(aInstrument.InstrumentName, DataModule1.RPCBroker1);

    if TempStr = '' then
      Exit;

    if TempStr = BAD_DATA then
      begin
        Result := BAD_DATA;
        Exit;
      end;

    for ii := 1 to NumberOfPieces(TempStr, RECORD_END) do
      begin
        SubStr := Piece(TempStr, RECORD_END, ii);

        if Pos('Scale', SubStr) > 0 then
          begin
            SubSubStr := Piece(SubStr, EQ, 2);
            if Piece(SubSubStr, U, 4) = AName then
              Result := Piece(SubSubStr, U, 1);
          end;
      end;
  end;

begin
  Result := OK;
  AScaleGroup := TScaleGroup.Create;
  AScale := TScale.Create;
  AKey := TKey.Create;


  for i := 2 to lvImportStrList.Count - 1 do
    begin

      // String with Scale Group data
      if Pos('ScaleGroup_', lvImportStrList[i]) = 1 then
        begin
          frmMHA_Progress.IncreaseProgressBar;
          j := 2;
 (*lvExportStrList.Add('ScaleGroup_' + IntToStr(i) + U +
            GroupName + U +
            IEN + U +
            OrdinateTitle + U +
            IntToStr(OrdinateMax) + U +
            IntToStr(OrdinateIncrement) + U +
            IntToStr(OrdinateMin) + U +
            IntToStr(Grid1Ordinate) + U +
            IntToStr(Grid2Ordinate) + U +
            IntToStr(Grid3Ordinate) + U +
            IntToStr(NumberOfScales) + U +
            IntToStr(Sequence));*)
          AScaleGroup.GroupName := Piece(lvImportStrList[i], U, j);
          Inc(j); // Ignore IEN  (used by Display Instrument)
          Inc(j);
          AScaleGroup.OrdinateTitle := Piece(lvImportStrList[i], U, j);
          Inc(j);
          AScaleGroup.OrdinateMax := StrToIntDef(Piece(lvImportStrList[i], U, j), 10);
          Inc(j);
          AScaleGroup.OrdinateIncrement := StrToIntDef(Piece(lvImportStrList[i], U, j), 1);
          Inc(j);
          AScaleGroup.OrdinateMin := StrToIntDef(Piece(lvImportStrList[i], U, j), 0);
          Inc(j);
          AScaleGroup.Grid1Ordinate := StrToIntDef(Piece(lvImportStrList[i], U, j), 0);
          Inc(j);
          AScaleGroup.Grid2Ordinate := StrToIntDef(Piece(lvImportStrList[i], U, j), 0);
          Inc(j);
          AScaleGroup.Grid3Ordinate := StrToIntDef(Piece(lvImportStrList[i], U, j), 0);
          Inc(j);
          AScaleGroup.NumberOfScales := StrToIntDef(Piece(lvImportStrList[i], U, j), 1);
          Inc(j);
          AScaleGroup.Sequence := StrToIntDef(Piece(lvImportStrList[i], U, j), 1);

          Reply := WriteScaleGroup(aInstrument.IEN, AScaleGroup,
            DataModule1.RPCBroker1);

          if reply <> OK then
            begin
              Result := Reply;
              AScaleGroup.Free;
              AScale.Free;
              AKey.Free;
              Exit;
            end
          else
            begin
              TempScaleGroupID := GetGroupScaleID(AScaleGroup.GroupName);
              if (TempScaleGroupID = '') or
                (TempScaleGroupID = BAD_DATA) then
                begin
                  Result := Reply;
                  AScaleGroup.Free;
                  AScale.Free;
                  AKey.Free;
                  Exit;
                end;
              AScaleGroup.IEN := TempScaleGroupID;
            end;

        end; // POS ScaleGroup

      // String with Scale data
      if Pos('Scale_', lvImportStrList[i]) = 1 then
        begin
          j := 2;
(*    lvExportStrList.Add('Scale_' + IntToStr(j) + U +
                        ScaleName + U +
                        IEN + U +
                        XLabel + U +
                        IntToStr(Sequence)); *)
          AScale.GroupIEN := AScaleGroup.IEN; // from above, not from file
          AScale.ScaleName := Piece(lvImportStrList[i], U, j);
          Inc(j);
          Inc(j); // Ignore Scale.IEN (used by Display Instrument)
          AScale.XLabel := Piece(lvImportStrList[i], U, j);
          Inc(j);
          AScale.Sequence := StrToIntDef(Piece(lvImportStrList[i], U, j), 0);

          reply := WriteScale(AScale, DataModule1.RPCBroker1);
          if reply <> OK then
            begin
              Result := Reply;
              Exit;
            end
          else
            begin
              TempScaleID := GetScaleID(AScale.ScaleName);
              if (TempScaleID = '') or
                (TempScaleID = BAD_DATA) then
                begin
                  Result := Reply;
                  AScaleGroup.Free;
                  AScale.Free;
                  AKey.Free;
                  Exit;
                end;
              AScale.IEN := TempScaleID;
            end;
        end;

      // String with Scoring Key data
      if Pos('ScoringKey_', lvImportStrList[i]) = 1 then
        begin
          j := 3;
(*
      lvExportStrList.Add('ScoringKey_' + IntToStr(k) + U +
                                    ScaleID + U +
                                    QuestionID + U +
                                    AnswerText + U +
                                    Value);*)
          AKey.ScaleID := AScale.IEN; // from above, not from file
          AKey.QuestionID := Piece(lvImportStrList[i], U, j);
          //AKey.QuestionID := ConvertOldToNew(Piece(lvImportStrList[i], U, j));
          Inc(j);
          AKey.AnswerText := Piece(lvImportStrList[i], U, j);
          Inc(j);
          AKey.Value := Piece(lvImportStrList[i], U, j);

          reply := WriteScoringKey(AKey, DataModule1.RPCBroker1);
          if reply <> OK then
            Result := Reply;
        end;
    end; // for i ...

  AScaleGroup.Free;
  AScale.Free;
  AKey.Free;
end;


function TImport.ImportReport(aInstrument: TInstrument): string;
// If there are any scripts associated with the report, they are footnoted
// to the report.
var
  i                           : Integer;
  ReportIEN,
  aReport                     : string;
  OKToCopy                    : Boolean;

begin
  Result := OK;

  aReport := '';
  ReportIEN := '';
  OkToCopy := False;

  for i := 2 to lvImportStrList.Count - 1 do
    begin
      if OKToCopy then
        begin
          frmMHA_Progress.IncreaseProgressBar;
          // Vista does not handle carriage returns well; mark with #124
          aReport := aReport + #124 + lvImportStrList[i];
        end;

      if Pos('Report_follows', lvImportStrList[i]) > 0 then
      begin
        OkToCopy := True;
         ReportIEN := Piece(lvImportStrList[i], U, 2);
        end;
      if Pos('End_Report', lvImportStrList[i]) > 0 then
        Break;
    end;

  if not OkToCopy then // No report
    Exit;

  // Replace the old QuestionIDs with the new ones
  (*
  for i := 0 to lvOldNewQuestionID.Count - 1 do
    begin
      aReport := StringReplace(aReport, Piece(lvOldNewQuestionID[i], EQ, 1),
        Piece(lvOldNewQuestionID[i], EQ, 2), [rfReplaceAll]);
    end; *)
ShowMessage('reportIEN '+reportIEN);
  if ReportIEN <> '' then
  Result := WriteReport(aReport, aInstrument.IEN, ReportIEN,
    DataModule1.RPCBroker1);
end;


function TImport.GetDisplayID(
  FontIn: TFont;
  AlignmentIn: TAlignment;
  LeftIn: Integer;
  IsAnswerDisplay: Boolean;
  Mask: string;
  Columns: Integer;
  Component: string): string;
// This will return the DisplayID for an existing Display, and if that
// doesn't exist, will save the Display and return the new ID.

var
  TempDisplay                 : TDisplay;

begin
  Result := NO_ID;

  TempDisplay := TDisplay.Create;

  with TempDisplay, Display_OPS do
    begin
      DisplayFont.Assign(FontIn);
      DisplayLeft := LeftIn;
      DisplayAlignment := AlignmentIn;
      if IsAnswerDisplay then
        begin
          DisplayMask := Mask;
          DisplayColumns := Columns;
          DisplayComponent := StrToAnswerComponent(Component);
        end;
    end; // with

  // Error messages will be presented upstream and a blank will be passed when
  // there is an error.
  Result := SearchDisplays(TempDisplay, IsAnswerDisplay);

  TempDisplay.Free;
end;

function TImport.SearchDisplays(WhichDisplay: TDisplay;
  IsAnswerDisplay: BOOLEAN): string;
var
  i                           : integer;
  reply                       : string;

  function DisplayMatch(ADisplay, BDisplay: TDisplay): string;
  begin
    Result := '';
    if (ADisplay.DisplayFont = BDisplay.DisplayFont) and
      (ADisplay.DisplayLeft = BDisplay.DisplayLeft) and
      (ADisplay.DisplayAlignment = BDisplay.DisplayAlignment) then
      begin
        if IsAnswerDisplay then
          begin
            if (ADisplay.DisplayMask = BDisplay.DisplayMask) and
              (ADisplay.DisplayColumns = BDisplay.DisplayColumns) and
              (ADisplay.DisplayComponent = BDisplay.DisplayComponent) then
              Result := BDisplay.DisplayID;
          end
        else
          Result := BDisplay.DisplayID;
      end;
  end;

begin
  reply := '';
  Result := '';

  //See if style already exists within this instrument's Display data...
  for i := 0 to lvInstrument.ListOfQuestions.Count - 1 do
    begin
      Reply := DisplayMatch(WhichDisplay,
        TQuestion(lvInstrument.ListOfQuestions[i]).Display);
      if Reply <> '' then
        break;
    end; // for i

  if reply = '' then
    begin
      for i := 0 to lvInstrument.ListOfQuestions.Count - 1 do
        begin
          Reply := DisplayMatch(WhichDisplay,
            TQuestion(lvInstrument.ListOfQuestions[i]).AnswerDisplay);
          if Reply <> '' then
            break;
        end; // for i
    end;

  if reply = '' then
    begin
      for i := 0 to lvInstrument.ListOfIntroductions.Count - 1 do
        begin
          Reply := DisplayMatch(WhichDisplay,
            TIntroduction(lvInstrument.ListOfIntroductions[i]).Display);
          if Reply <> '' then
            break;
        end; // for i
    end;

  if reply = '' then
    begin
      for i := 0 to lvInstrument.ListOfSections.Count - 1 do
        begin
          Reply := DisplayMatch(WhichDisplay,
            TIntroduction(lvInstrument.ListOfSections[i]).Display);
          if Reply <> '' then
            break;
        end; // for i
    end;

  // DisplayID found; finished
  if reply <> '' then
    begin
      Result := Reply;
      Exit;
    end;

  // New Display, get ID
  if reply = '' then
    begin
      result := WriteDisplay(WhichDisplay, IsAnswerDisplay, DataModule1.RPCBroker1);
      //   If there is an error, this line will be passed:
      // 'ERROR: YTQ SAVE: ' + FILE_DISPLAY + ' FAILED.' + U + Results[1];
      if Pos(Result, 'ERROR') > 0 then
        begin
          ShowMessage('Error saving a display: ' + CRLF + CRLF + Result);
          Result := '';
        end;
    end;
end;

procedure TImport.WriteUpdateSeries(aSeries: string);
begin

  with DataModule1.RPCBroker1 do
    begin

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ SAVE';
      Param[0].Mult['"FILEN"'] := FILE_TESTS_AND_SURVEYS;

      Param[0].Mult['"1"'] := '.01^WAS';
      Param[0].Mult['"17"'] := '9^' + aSeries;

      Param[0].PType := List;
      Call;

    end; //with datamodule
end;


end.

